@@ -31,6 +31,9 @@
 #include <stdarg.h>		/* C89 */
 #include <stdbool.h>		/* Gnulib, C99 */
 #include <signal.h>		/* ? */
+#include "gettext.h"		/* Gnulib/gettext */
+#define _(s) gettext(s)
+#define N_(s) gettext_noop(s)
 #include "xalloc.h"		/* Gnulib */
 #include "version-etc.h"	/* Gnulib */
 #include "intutil.h"
@@ -47,7 +50,7 @@ enum {
     OPT_VERSION,
 };
 
-#define SHORT_OPTIONS "bvi:o:p:"
+#define SHORT_OPTIONS "bv::i:o:p:"
 
 static struct option long_options[] = {
 #ifdef HAVE_ID3LIB
@@ -61,13 +64,13 @@ static struct option long_options[] = {
     { "no-timestamp", no_argument, NULL, OPT_NOTIMESTAMP },
     { "timestamp", optional_argument, NULL, OPT_TIMESTAMP },
     { "interface", required_argument, NULL, 'i' },
-    { "verbose", no_argument, NULL, 'v' },
+    { "verbose", optional_argument, NULL, 'v' },
     { "help", no_argument, NULL, OPT_HELP },
     { "version", no_argument, NULL, OPT_VERSION },
     { NULL, 0, NULL, 0 }
 };
 
-const char version_etc_copyright[] = "Copyright (C) 2004 Oskar Liljeblad.";
+const char version_etc_copyright[] = "Copyright (C) 2005 Oskar Liljeblad.";
 
 static void
 dummy_signal_handler(int signal)
@@ -75,6 +78,26 @@ dummy_signal_handler(int signal)
     /* No operation */
 }
 
+static void
+write_pid_file(const char *pidfilename, pid_t pid, bool background)
+{
+    if (pidfilename != NULL) {
+	FILE *pidfh;
+
+	/* If we are going to background, we don't want to log this message
+         * to the console. We only want to tell if the pid file failed to
+         * write, which is pretty serious though.
+         */
+	if (!background)
+	    say(2, _("Writing pid to pid file `%s'\n"), pidfilename);
+	if ((pidfh = fopen(pidfilename, "w")) == NULL)
+	    die(_("Cannot open pid file `%s' for writing - %s\n"), pidfilename, errstr);
+	fprintf(pidfh, "%d", getpid());
+	if (fclose(pidfh) != 0)
+	    die(_("Cannot close pid file `%s' - %s\n"), pidfilename, errstr);
+    }
+}
+
 int
 main(int argc, char **argv)
 {
@@ -85,8 +108,10 @@ main(int argc, char **argv)
     uint16_t listenport = 0;
     bool background = false;
     bool timestamp_specified = false;
+    char *logfilename = NULL;
+    char *timestamp_format = NULL;
 
-    while (1) {
+    for (;;) {
     	int c;
 
         c = getopt_long(argc, argv, SHORT_OPTIONS, long_options, NULL);
@@ -96,12 +121,10 @@ main(int argc, char **argv)
         switch (c) {
         case 'i':
 	    if ((listenip = get_ip_by_spec(optarg)) == NULL)
-	    	die("invalid address or interface name `%s'\n", optarg);
+	    	die(_("invalid address or interface name `%s'\n"), optarg);
             break;
         case 'o':
-            if ((logfh = fopen(optarg, "a")) == NULL)
-                die("cannot open log file `%s' for appending\n", optarg);
-	    setlinebuf(logfh);
+            logfilename = optarg;
             break;
 #ifdef HAVE_ID3LIB
 	case OPT_DISABLE_ID3:
@@ -110,7 +133,7 @@ main(int argc, char **argv)
 #endif
 	case OPT_FRIENDLY_NAME:
 	    if (optarg[0] == '\0')
-		die("friendly name cannot be empty\n");
+		die(_("friendly name cannot be empty\n"));
 	    friendly_name = xstrdup(optarg);
 	    break;
 	case OPT_PIDFILE:
@@ -118,7 +141,7 @@ main(int argc, char **argv)
 	    break;
 	case 'p':
 	    if (!parse_uint16(optarg, &listenport))
-		die("invalid port number `%s'\n", optarg);
+		die(_("invalid port number `%s'\n"), optarg);
 	    break;
 	case OPT_NOTIMESTAMP:
 	    timestamp_format = NULL;
@@ -136,42 +159,39 @@ main(int argc, char **argv)
 		if (optarg[0] == 'v') {
 		    for (verbosity = 0; optarg[verbosity] != '\0'; verbosity++) {
 			if (optarg[verbosity] != 'v')
-			    die("invalid verbosity `%s'\n", optarg);
+			    die(_("invalid verbosity `%s'\n"), optarg);
 		    }
 		    verbosity++;
 		} else {
 		    if (!parse_uint32(optarg, &verbosity))
-			die("invalid verbosity `%s'\n", optarg);
+			die(_("invalid verbosity `%s'\n"), optarg);
 		}
 		if (verbosity > MAX_VERBOSITY)
-		    die("maximum verbosity is %d\n", MAX_VERBOSITY);
+		    die(_("maximum verbosity is %d\n"), MAX_VERBOSITY);
 	    } else {
-		verbosity = 1;
+		verbosity++;
 	    }
             break;
         case OPT_HELP:
-            printf("Usage: %s [OPTIONS]... DIRECTORY\n", argv[0]);
-            printf("Run the UPnP media server.\n");
-            printf("\n");
-	    printf("      --friendly-name=NAME  set display name for media server\n");
+            printf(_("Usage: %s [OPTIONS]... DIRECTORIES...\n"), argv[0]);
+            printf(_("Run the UPnP media server.\n\n"));
+	    printf(_("      --friendly-name=NAME  set display name for media server\n"));
 #ifdef HAVE_ID3LIB
-	    printf("      --disable-id3         do not scan ID3 information on files\n");
+	    printf(_("      --disable-id3         do not scan files for ID3 information\n"));
 #endif
-	    printf("  -v, --verbose[=LEVEL]     set verbosity level (0-4)\n");
-            printf("      --pid-file=FILE       write pid to FILE when up and running\n");
-            printf("  -i, --interface=NAME      listen on a specific interface\n");
-            printf("  -p, --port=PORT           listen on a specific port\n");
-            printf("  -o, --output=LOGFILE      file for logging\n");
-            printf("  -b, --background          go to background (detach)\n");
-            printf("      --no-timestamp        do not prepend timestamp to log entries\n");
-            printf("      --timestamp[=FORMAT]  prepend timestamp with optional time format\n");
-            printf("      --help                display this help and exit\n");
-            printf("      --version             output version information and exit\n");
-            printf("\n");
-	    printf("The directory argument specifies where audio files are located.\n");
-            printf("See strftime(3) for description of the timestamp format.\n");
-	    printf("\n");
-            printf("Report bugs to <%s>.\n", PACKAGE_BUGREPORT);
+	    printf(_("  -v, --verbose[=LEVEL]     set verbosity level (0-4)\n"));
+            printf(_("      --pid-file=FILE       write pid to FILE when up and running\n"));
+            printf(_("  -i, --interface=NAME      listen on a specific interface\n"));
+            printf(_("  -p, --port=PORT           listen on a specific port\n"));
+            printf(_("  -o, --output=LOGFILE      file for logging\n"));
+            printf(_("  -b, --background          go to background (detach)\n"));
+            printf(_("      --no-timestamp        do not prepend timestamp to log entries\n"));
+            printf(_("      --timestamp[=FORMAT]  prepend timestamp with optional time format\n"));
+            printf(_("      --help                display this help and exit\n"));
+            printf(_("      --version             output version information and exit\n"));
+	    printf(_("\nThe directories arguments specify where audio files are located.\n"
+		     "See strftime(3) for description of the timestamp format.\n"));
+            printf(_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
             exit(EXIT_SUCCESS);
         case OPT_VERSION:
             version_etc(stdout, NULL, PACKAGE, VERSION, "Oskar Liljeblad", NULL);
@@ -181,26 +201,25 @@ main(int argc, char **argv)
         }
     }
 
-    if (argc - optind <= 0)
-	die("missing directory argument\nTry `%s --help' for more information.\n", argv[0]);
-    if (argc - optind > 1)
-	die("too many arguments\nTry `%s --help' for more information.\n", argv[0]);
-    rootpath = argv[optind];
-
-    if (listenip != NULL)
-	say(2, "Using IP address `%s'.\n", listenip);
-
-    if (!timestamp_specified && logfh != NULL)
-    	timestamp_format = DEFAULT_TIMESTAMP_FORMAT;
+    if (argc - optind <= 0) {
+        warn(_("missing directory argument\n"));
+        fprintf(stderr, _("Try `%s --help' for more information.\n"), argv[0]);
+        exit(1);
+    }
+    if (!timestamp_specified && logfilename != NULL)
+        timestamp_format = DEFAULT_TIMESTAMP_FORMAT;
 
     if (background) {
         pid_t pid;
 
         /*say(2, "Moving process into background...\n");*/
     	if ((pid = fork()) < 0)
-    	    die("Cannot fork new process - %s\n", errstr);
-        if (pid != 0)
+    	    die(_("Cannot fork new process - %s\n"), errstr);
+        if (pid != 0) {
+            /* Write pid before we return control to the shell. */
+            write_pid_file(pidfilename, pid, true);
             exit(EXIT_SUCCESS);
+        }
 
         fclose(stdin);	    	/* ignore errors */
 	fclose(stdout);     	/* ignore errors */
@@ -210,66 +229,63 @@ main(int argc, char **argv)
         close(STDERR_FILENO);	/* ignore errors */
 
         if (setsid() < 0)
-	    die("Cannot create process session - %s\n", errstr);
+	    die(_("Cannot create process session - %s\n"), errstr);
 
-	/* XXX: The following is to work around a bug in 
+	/* XXX: The following is necessary to work around a bug in
 	 *      libupnp 1.2.1 - fd 0 must not be closed!
 	 */
 	if (open("/dev/null", O_RDONLY) < 0)
-	    die("Cannot open /dev/null - %s\n", errstr);
+	    die(_("Cannot open /dev/null - %s\n"), errstr);
     }
 
+    if (!timestamp_specified && logfilename != NULL)
+    	timestamp_format = DEFAULT_TIMESTAMP_FORMAT;
+
+    init_logging(logfilename, timestamp_format);
+
+    /* We could write pid before initiating logging too.
+     */
+    if (!background)
+        write_pid_file(pidfilename, getpid(), false);
+
     signalaction.sa_handler = dummy_signal_handler;
     if (sigemptyset(&signalaction.sa_mask) < 0)
-    	die("Cannot empty signal set - %s\n", errstr);
+    	die(_("Cannot empty signal set - %s\n"), errstr);
     signalaction.sa_flags = SA_RESTART;
     signalaction.sa_restorer = NULL;
     if (sigaction(SIGINT, &signalaction, NULL) < 0)
-    	die("Cannot register signal handler - %s\n", errstr);
+    	die(_("Cannot register signal handler - %s\n"), errstr);
     if (sigaction(SIGTERM, &signalaction, NULL) < 0)
-    	die("Cannot register signal handler - %s\n", errstr);
+    	die(_("Cannot register signal handler - %s\n"), errstr);
     if (sigaction(SIGUSR1, &signalaction, NULL) < 0)
-    	die("Cannot register signal handler - %s\n", errstr);
+    	die(_("Cannot register signal handler - %s\n"), errstr);
 
     if (sigemptyset(&signalset) < 0)
-    	die("Cannot empty signal set - %s\n", errstr);
+    	die(_("Cannot empty signal set - %s\n"), errstr);
     if (sigaddset(&signalset, SIGTERM) < 0)
-    	die("Cannot add signal to set - %s\n", errstr);
+    	die(_("Cannot add signal to set - %s\n"), errstr);
     if (sigaddset(&signalset, SIGINT) < 0)
-    	die("Cannot add signal to set - %s\n", errstr);
+    	die(_("Cannot add signal to set - %s\n"), errstr);
     if (sigaddset(&signalset, SIGUSR1) < 0)
-    	die("Cannot add signal to set - %s\n", errstr);
+    	die(_("Cannot add signal to set - %s\n"), errstr);
     if (sigprocmask(SIG_BLOCK, &signalset, NULL) < 0)
-    	die("Cannot block signals - %s\n", errstr);
+    	die(_("Cannot block signals - %s\n"), errstr);
 
-    if (!init_metadata()) {
-	if (logfh != NULL)
-	    fclose(logfh);	/* ignore errors */
+    if (!init_metadata(argv+optind, argc-optind)) {
+        finish_logging(false);
 	exit(EXIT_FAILURE);
     }
 
-    log_starting();
-    
-    if (pidfilename != NULL) {
-	FILE *pidfh;
-	say(2, "Writing pid to pid file `%s'\n", pidfilename);
-	if ((pidfh = fopen(pidfilename, "w")) == NULL)
-	    die("Cannot open pid file `%s' for writing - %s\n", pidfilename, errstr);
-	fprintf(pidfh, "%d", getpid());
-	if (fclose(pidfh) != 0)
-	    die("Cannot close pid file `%s' - %s\n", pidfilename, errstr);
-    }
-
     init_upnp(listenip, listenport);
 
     while (true) {
     	int sig;
       	
       	if ((sig = TEMP_FAILURE_RETRY(sigwaitinfo(&signalset, NULL))) < 0)
-    	    die("sigwaitinfo failed - %s\n", errstr);
-      	say(2, "Received signal %d\n", sig);
+    	    die(_("sigwaitinfo failed - %s\n"), errstr);
+      	say(2, _("Received signal %d\n"), sig);
       	if (sig == SIGUSR1) {
-      	    //say(1, "Reloading configuration file...\n"); /* XXX: not implemented */
+      	    /*say(1, "Reloading configuration file...\n");*/ /* XXX: not implemented */
       	}
       	if (sig == SIGTERM || sig == SIGINT)
             break;
@@ -281,10 +297,7 @@ main(int argc, char **argv)
     if (pidfilename != NULL)
     	unlink(pidfilename);	/* ignore errors */
 
-    say(2, "%s process id %d terminated normally\n", PACKAGE, getpid());
-
-    if (logfh != NULL)
-        fclose(logfh);	/* ignore errors */
+    finish_logging(true);
 
     exit(EXIT_SUCCESS);
 }
