@@ -19,17 +19,26 @@
  */
 
 #include <config.h>
+#include <config.h>
+#include <id3.h>                /* id3lib */
+#undef false /* id3lib bug - it shouldn't mess with false and true! */
+#undef true
+#include <stdbool.h>            /* Gnulib, C99 */
 #include <stdint.h>		/* Gnulib, C99 */
-#include <dirent.h>		/* ? */
-#include <sys/stat.h>		/* ? */
-#include <sys/types.h>		/* ? */
-#include <fcntl.h>		/* ? */
+#include <dirent.h>		/* POSIX */
+#include <sys/stat.h>		/* POSIX */
+#include <sys/types.h>		/* POSIX */
+#include <fcntl.h>		/* POSIX */
 #include <unistd.h>		/* POSIX */
 #include <ctype.h>		/* C89 */
-#include <id3.h>		/* id3lib */
+#include "gettext.h"		/* Gnulib/gettext */
+#define _(s) gettext(s)
+#define N_(s) gettext_noop(s)
 #include "full-read.h"		/* Gnulib */
 #include "xalloc.h"		/* Gnulib */
 #include "xvasprintf.h"		/* Gnulib */
+#include "dirname.h"		/* Gnulib */
+#include "xstrndup.h"		/* Gnulib */
 #include "strbuf.h"
 #include "gmediaserver.h"
 
@@ -42,7 +51,6 @@ typedef enum {
     FILE_WMA
 } FileType;
 
-char *rootpath = NULL;
 static Entry *root_entry;
 static uint32_t entry_count = 0;
 static uint32_t entries_size = 0;
@@ -85,23 +93,57 @@ concat_filenames(const char *p1, const char *p2)
     return out;
 }
 
+EntryDetail *
+get_entry_detail(Entry *entry, EntryDetailType type)
+{
+    EntryDetail *d;
+    
+    for (d = entry->details; d != NULL; d = d->next) {
+	if (d->type == type)
+	    return d;
+    }
+
+    return NULL;
+}
+
+bool
+has_entry_detail(Entry *entry, EntryDetailType type)
+{
+    return get_entry_detail(entry, type) != NULL;
+}
+
 char *
 get_entry_path(Entry *entry)
 {
     StrBuf *path;
-    char *fullpath;
+    char *full_path;
+    char *local_path;
 
     path = strbuf_new();
-    while (entry->id != 0) {
+    while (!has_entry_detail(entry, DETAIL_LOCAL_PATH)) {
 	strbuf_prepend(path, entry->filename);
 	if (entry->parent != 0)
 	    strbuf_prepend(path, "/");
 	entry = entries[entry->parent];
     }
 
-    fullpath = concat_filenames(rootpath, strbuf_buffer(path));
+    local_path = get_entry_detail(entry, DETAIL_LOCAL_PATH)->data.local_path;
+    full_path = concat_filenames(local_path, strbuf_buffer(path));
     strbuf_free(path);
-    return fullpath;
+    return full_path;
+}
+
+static EntryDetail *
+add_entry_detail(Entry *entry, EntryDetailType type)
+{
+    EntryDetail *detail;
+
+    detail = xmalloc(sizeof(EntryDetail));
+    detail->next = entry->details;
+    detail->type = type;
+    entry->details = detail;
+
+    return detail;
 }
 
 static Entry*
@@ -111,14 +153,16 @@ make_entry(const char *name, int32_t parent, bool directory)
 
     entry = xmalloc(sizeof(Entry));
     entry->id = entry_count++;
-    entry->child_count = directory ? 0 : -1;
-    entry->children = NULL;
     entry->parent = parent;
     entry->filename = xstrdup(name);
-    entry->title = NULL;
-    entry->artist = NULL;
-    entry->album = NULL;
-    entry->genre = NULL;
+    entry->details = NULL;
+
+    if (directory) {
+	EntryDetail *detail;
+	detail = add_entry_detail(entry, DETAIL_CHILDREN);
+	detail->data.children.count = 0;
+	detail->data.children.list = NULL;
+    }
 
     if (entry->id >= entries_size) {
         entries_size *= 2;
@@ -145,18 +189,18 @@ check_file_content_type(const char *fullpath)
     uint8_t buf[4];
     size_t count;
 
-    say(4, "Check content type of file %s\n", fullpath);
+    say(4, _("Check content type of file %s\n"), fullpath);
 
     fd = open(fullpath, O_RDONLY);
     if (fd < 0) {
-	warn("%s: cannot open for reading: %s\n", fullpath, errstr);
+	warn(_("%s: cannot open for reading: %s\n"), fullpath, errstr);
 	return FILE_UNKNOWN;
     }
 
     count = full_read(fd, buf, 4);
     if (count < 4) {
 	if (errno != 0)
-	    warn("%s: cannot read: %s\n", fullpath, errstr);
+	    warn(_("%s: cannot read: %s\n"), fullpath, errstr);
 	close(fd); /* Ignore errors since we opened for reading */
 	return FILE_UNKNOWN;
     }
@@ -164,19 +208,19 @@ check_file_content_type(const char *fullpath)
     close(fd); /* Ignore errors since we opened for reading */
     
     if (buf[0] == 0xFF && (buf[1] & 0xFE) == 0xFA) {
-	say(4, "Matched type MPEG ADTS for %s\n", fullpath);
+	say(4, _("Matched type MPEG ADTS for %s\n"), fullpath);
 	return FILE_MPEG_ADTS;
     }
     if (buf[0] == 'I' && buf[1] == 'D' && buf[2] == '3') {
-	say(4, "Matched type MP3 with ID3 tag for %s\n", fullpath);
+	say(4, _("Matched type MP3 with ID3 tag for %s\n"), fullpath);
 	return FILE_MP3_ID3;
     }
     if (buf[0] == 0x30 && buf[1] == 0x26 && buf[2] == 0xb2 && buf[3] == 0x75) {
-	say(4, "Matched type WMA for %s\n", fullpath);
+	say(4, _("Matched type WMA for %s\n"), fullpath);
 	return FILE_WMA;
     }
 
-    say(4, "%s: skipping file - unrecognized content type\n", fullpath);
+    say(4, _("%s: skipping file - unrecognized content type\n"), fullpath);
     return FILE_UNKNOWN;
 }
 
@@ -211,28 +255,30 @@ scan_entry(const char *fullpath, const char *name, int32_t parent)
     struct stat sb;
 
     if (stat(fullpath, &sb) < 0) {
-        warn("%s: cannot stat: %s\n", fullpath, errstr);
+        warn(_("%s: cannot stat: %s\n"), fullpath, errstr);
         return NULL;
     }
 
     if (S_ISDIR(sb.st_mode)) {
         Entry *entry;
+	EntryDetail *detail;
 	int32_t *children;
+	uint32_t child_count;
         struct dirent **dirents;
         int dirent_count;
         int c;
 
-	say(4, "Scanning directory %s\n", fullpath);
+	say(4, _("Scanning directory %s\n"), fullpath);
         dirent_count = scandir(fullpath, &dirents, NULL, NULL);
         if (dirent_count < 0) {
-            warn("%s: cannot scan directory: %s\n", fullpath, errstr);
+            warn(_("%s: cannot scan directory: %s\n"), fullpath, errstr);
             return NULL;
         }
 
         entry = make_entry(name, parent, true);
 	children = xmalloc(sizeof(int32_t) * dirent_count);
 
-	entry->child_count = 0;
+	child_count = 0;
         for (c = 0; c < dirent_count; c++) {
             if (strcmp(dirents[c]->d_name, ".") != 0 && strcmp(dirents[c]->d_name, "..") != 0) {
 		Entry *child;
@@ -241,12 +287,14 @@ scan_entry(const char *fullpath, const char *name, int32_t parent)
 		child_path = concat_filenames(fullpath, dirents[c]->d_name);
 		child = scan_entry(child_path, dirents[c]->d_name, entry->id);
 		if (child != NULL)
-		    children[entry->child_count++] = child->id;
+		    children[child_count++] = child->id;
 		free(child_path);
 	    }
         }
 
-	entry->children = xmemdup(children, sizeof(int32_t) * entry->child_count);
+	detail = get_entry_detail(entry, DETAIL_CHILDREN);
+	detail->data.children.count = child_count;
+	detail->data.children.list = xmemdup(children, sizeof(int32_t) * child_count);
 	free(children);
 
         return entry;
@@ -254,51 +302,93 @@ scan_entry(const char *fullpath, const char *name, int32_t parent)
 
     if (S_ISREG(sb.st_mode)) {
 	Entry *entry;
+	EntryDetail *detail;
 	FileType type;
 
         if (parent == -1) {
-            warn("%s: root must be a directory\n", fullpath);
+            warn(_("%s: root must be a directory\n"), fullpath);
             return NULL;
         }
 	type = check_file_content_type(fullpath);
 	if (type == FILE_UNKNOWN)
 	    return NULL;
 
-	say(4, "Adding file %s\n", fullpath);
+	say(4, _("Adding file %s\n"), fullpath);
 	entry = make_entry(name, parent, false);
 
 #ifdef HAVE_ID3LIB
 	if (id3_enabled && (type == FILE_MPEG_ADTS || type == FILE_MP3_ID3)) {
 	    ID3Tag_Clear(id3);
 	    ID3Tag_Link(id3, fullpath);
-	    entry->title = get_id3_string(id3, ID3FID_TITLE);
-	    entry->album = get_id3_string(id3, ID3FID_ALBUM);
-	    entry->artist = get_id3_string(id3, ID3FID_LEADARTIST);
-	    entry->genre = get_id3_string(id3, ID3FID_CONTENTTYPE);
+
+	    detail = add_entry_detail(entry, DETAIL_TAG);
+	    detail->data.tag.title = get_id3_string(id3, ID3FID_TITLE);
+	    detail->data.tag.album = get_id3_string(id3, ID3FID_ALBUM);
+	    detail->data.tag.artist = get_id3_string(id3, ID3FID_LEADARTIST);
+	    detail->data.tag.genre = get_id3_string(id3, ID3FID_CONTENTTYPE);
 	}
 #endif
 	
         return entry;
     }
 
-    warn("%s: skipping file - unsupported file type\n", fullpath);
+    warn(_("%s: skipping file - unsupported file type\n"), fullpath);
     return NULL;
 }
 
 bool
-init_metadata(void)
+init_metadata(char **pathv, int pathc)
 {
     entries_size = DEFAULT_ENTRIES_SIZE;
     entries = xmalloc(sizeof(Entry *) * entries_size);
+
 #ifdef HAVE_ID3LIB
     if (id3_enabled)
 	id3 = ID3Tag_New();
 #endif
-    root_entry = scan_entry(rootpath, "", -1);
+
+    if (pathc > 1) {
+	int32_t *children;
+	EntryDetail *detail;
+	uint32_t c;
+
+	root_entry = make_entry("", -1, true);
+	children = xmalloc(sizeof(int32_t) * pathc);
+	for (c = 0; c < pathc; c++) {
+	    Entry *entry;
+	    char *name;
+	    uint32_t d;
+
+	    name = base_name(pathv[c]);
+	    if (*name == '\0' || *name == '/') {
+	        name = xstrdup(pathv[c]);
+            } else {
+	        for (d = strlen(name)-1; name[d] == '/'; d--);
+	        name = xstrndup(name, d+1);
+            }
+
+	    entry = scan_entry(pathv[c], name, -1);
+	    children[c] = entry->id;
+	    free(name);
+	    detail = add_entry_detail(entry, DETAIL_LOCAL_PATH);
+	    detail->data.local_path = xstrdup(pathv[c]);
+	}
+	detail = get_entry_detail(root_entry, DETAIL_CHILDREN);
+	detail->data.children.count = pathc;
+	detail->data.children.list = children;
+    } else {
+	EntryDetail *detail;
+	
+	root_entry = scan_entry(pathv[0], "", -1);
+	detail = add_entry_detail(root_entry, DETAIL_LOCAL_PATH);
+	detail->data.local_path = xstrdup(pathv[0]);
+    }
+
 #ifdef HAVE_ID3LIB
     if (id3_enabled)
 	ID3Tag_Delete(id3);
 #endif
+
     return root_entry != NULL;
 }
 
@@ -308,11 +398,29 @@ finish_metadata(void)
     uint32_t c;
 
     for (c = 0; c < entry_count; c++) {
-	free(entries[c]->children);
-	free(entries[c]->title);
-	free(entries[c]->artist);
-	free(entries[c]->album);
-	free(entries[c]->genre);
+	EntryDetail *d = entries[c]->details;
+
+	while (d != NULL) {
+	    EntryDetail *next;
+
+	    if (d->type == DETAIL_CHILDREN) {
+		free(d->data.children.list);
+	    }
+	    else if (d->type == DETAIL_LOCAL_PATH) {
+		free(d->data.local_path);
+	    }
+	    else if (d->type == DETAIL_TAG) {
+		free(d->data.tag.title);
+		free(d->data.tag.artist);
+		free(d->data.tag.album);
+		free(d->data.tag.genre);
+	    }
+
+	    next = d->next;
+	    free(d);
+	    d = next;
+	}
+
 	free(entries[c]->filename);
         free(entries[c]);
     }
