@@ -25,11 +25,15 @@
 #include <assert.h>		/* C89 */
 #include <netinet/in.h>		/* ?; inet_ntoa */
 #include <arpa/inet.h>		/* ?; inet_ntoa */
+#include "gettext.h"            /* Gnulib/gettext */
+#define _(s) gettext(s)
+#define N_(s) gettext_noop(s)
+#include "quotearg.h"		/* Gnulib */
 #include "xvasprintf.h"		/* Gnulib */
 #include "xgethostname.h"	/* Gnulib */
 #include "xalloc.h"		/* Gnulib */
-#include "gmediaserver.h"
 #include "intutil.h"
+#include "gmediaserver.h"
 #include "web/MediaServer.h"
 
 char *friendly_name = NULL;
@@ -54,7 +58,7 @@ upnp_errmsg(int res)
 {
     return UpnpGetErrorMessage(res);
 
-/*    switch (res) {
+/*  switch (res) {
     case UPNP_E_SUCCESS:
     	return "Success";   	    	    // strerror(ENOMEM);
     case UPNP_E_INVALID_HANDLE:
@@ -129,14 +133,14 @@ upnp_errmsg(int res)
 static void
 handle_subscription_request(struct Upnp_Subscription_Request *event)
 {
-    say(2, "Event received: Subscription request\n");
+    say(2, _("Event received: Subscription request\n"));
     /* XXX: not implemented */
 }
 
 static void
 handle_get_var_request(struct Upnp_State_Var_Request *event)
 {
-    say(2, "Event received: Get variable request\n");
+    say(2, _("Event received: Get variable request\n"));
     /* XXX: not implemented */
 }
 
@@ -172,13 +176,13 @@ handle_action_request(struct Upnp_Action_Request *request)
     Service *service;
     ServiceAction *action;
 
-    say(2, "Event received: Action request\n");
-    say(3, "Event device UDN: %s\n", request->DevUDN);
-    say(3, "Event service ID: %s\n", request->ServiceID);
-    say(3, "Event action name: %s\n", request->ActionName);
-    if (request->ErrCode != UPNP_E_SUCCESS)
-	say(3, "Event error: %s (%d)\n", upnp_errmsg(request->ErrCode), request->ErrCode);
-    say(3, "Event source: %s\n", inet_ntoa(request->CtrlPtIPAddr));
+    say(2, _("Event received: Action request\n"));
+    say(3, _("Event device UDN: %s\n"), quotearg(request->DevUDN));
+    say(3, _("Event service ID: %s\n"), quotearg(request->ServiceID));
+    say(3, _("Event action name: %s\n"), quotearg(request->ActionName));
+    /*if (request->ErrCode != UPNP_E_SUCCESS)
+	say(3, _("Event error: %s (%d)\n"), upnp_errmsg(request->ErrCode), request->ErrCode);*/
+    say(3, _("Event source: %s\n"), inet_ntoa(request->CtrlPtIPAddr));
 
     if (verbosity >= 4) {
         DOMString dump;
@@ -186,18 +190,18 @@ handle_action_request(struct Upnp_Action_Request *request)
         char *line2;
 
         dump = ixmlPrintDocument(request->ActionRequest);
-        say(4, "Event action request:\n");
+        say(4, _("Event action request:\n"));
         for (line = dump; (line2 = strstr(line, "\n")) != NULL; line = line2+1) {
             *line2 = '\0';
-            say(4, "  %s\n", line);
+            say(4, "  %s\n", quotearg(line));
         }
 	if (*line)
-            say(4, "%s\n", line);
+            say(4, "%s\n", quotearg(line));
         ixmlFreeDOMString(dump);
     }
 
     if (strcmp(request->DevUDN, DEVICE_UDN) != 0) {
-        say(1, "Discarding event - event device UDN not recognized\n");
+        say(1, _("Discarding event - event device UDN (%s) not recognized\n"), quotearg(request->DevUDN));
         return;
     }
 
@@ -210,14 +214,13 @@ handle_action_request(struct Upnp_Action_Request *request)
 	if (action->function(&event) && event.status) {
 	    request->ErrCode = UPNP_E_SUCCESS;
 
-
 	    if (verbosity >= 4) {
 		DOMString dump;
 		char *line;
 		char *line2;
-		
+
 		dump = ixmlPrintDocument(request->ActionResult);
-		say(4, "Event action result:\n");
+		say(4, _("Event action result:\n"));
 		for (line = dump; (line2 = strstr(line, "\n")) != NULL; line = line2+1) {
 		    *line2 = '\0';
 		    say(4, "  %s\n", line);
@@ -227,19 +230,21 @@ handle_action_request(struct Upnp_Action_Request *request)
 		ixmlFreeDOMString(dump);
 	    }
 
-	    say(2, "Action executed successfully\n");
+	    say(2, _("Action executed successfully\n"));
 	} else {
-	    say(2, "Action failed\n");
+	    say(2, _("Action failed.\n"));
+	    say(3, _("Error code: %d\n"), event.request->ErrCode);
+	    say(3, _("Error message: %s\n"), event.request->ErrStr);
 	}
 	return;
     }
     
     if (service != NULL) {
-	say(1, "Discarding event - action name not recognized\n");
-	strcpy(request->ErrStr, "Unrecognized action");
+	say(1, _("Discarding event - action name (%s) not recognized\n"), quotearg(request->ActionName));
+	strcpy(request->ErrStr, _("Unrecognized action"));
     } else {
-	say(1, "Discarding event - service ID not recognized\n");
-	strcpy(request->ErrStr, "Unrecognized service");
+	say(1, _("Discarding event - service ID (%s) not recognized\n"), quotearg(request->ServiceID));
+	strcpy(request->ErrStr, _("Unrecognized service"));
     }
     request->ActionResult = NULL;
     request->ErrCode = UPNP_SOAP_E_INVALID_ACTION;
@@ -259,7 +264,7 @@ device_callback_event_handler(Upnp_EventType eventtype, void *event, void *cooki
         handle_action_request((struct Upnp_Action_Request *) event);
         break;
     default:
-        say(2, "Received an unknown event (type 0x%x)\n", eventtype);
+        say(2, _("Received an unknown event (type 0x%x)\n"), eventtype);
         break;
     }
 
@@ -267,7 +272,7 @@ device_callback_event_handler(Upnp_EventType eventtype, void *event, void *cooki
 }
 
 void
-upnp_set_error(ActionEvent *event, const char *format, ...)
+upnp_set_error(ActionEvent *event, int error_code, const char *format, ...)
 {
     va_list ap;
     
@@ -287,11 +292,12 @@ upnp_get_i4(ActionEvent *event, const char *key)
 
     value = upnp_get_string(event, key);
     if (value == NULL)
-	return 0;
+	return 0; /* upnp_get_string will have set error details */
     if (parse_int32(value, &out))
 	return out;
 
-    upnp_set_error(event, "Invalid value for %s argument (%s)", key, value);
+    upnp_set_error(event, UPNP_SOAP_E_INVALID_ARGS,
+        _("Invalid value for %s argument (%s) - expected 32-bit signed integer"), key, quotearg(value));
     return 0;
 }
 
@@ -303,11 +309,12 @@ upnp_get_ui4(ActionEvent *event, const char *key)
 
     value = upnp_get_string(event, key);
     if (value == NULL)
-	return 0;
+	return 0; /* upnp_get_string will have set error details */
     if (parse_uint32(value, &out))
 	return out;
 
-    upnp_set_error(event, "Invalid value for %s argument (%s)", key, value);
+    upnp_set_error(event, UPNP_SOAP_E_INVALID_ARGS,
+        _("Invalid value for %s argument (%s) - expected 32-bit unsigned integer"), key, quotearg(value));
     return 0;
 }
 
@@ -318,12 +325,14 @@ upnp_get_string(ActionEvent *event, const char *key)
     
     node = (IXML_Node *) event->request->ActionRequest;
     if (node == NULL) {
-	upnp_set_error(event, "Invalid action request document");
+	upnp_set_error(event, UPNP_SOAP_E_INVALID_ARGS,
+	    _("Invalid action request document"));
 	return NULL;
     }
     node = ixmlNode_getFirstChild(node);
     if (node == NULL) {
-	upnp_set_error(event, "Invalid action request document");
+	upnp_set_error(event, UPNP_SOAP_E_INVALID_ARGS,
+	    _("Invalid action request document"));
 	return NULL;
     }
     node = ixmlNode_getFirstChild(node);
@@ -339,7 +348,8 @@ upnp_get_string(ActionEvent *event, const char *key)
 	}
     }
 
-    upnp_set_error(event, "Missing action request argument (%s)", key);
+    upnp_set_error(event, UPNP_SOAP_E_INVALID_ARGS,
+        _("Missing action request argument (%s)"), key);
     return NULL;
 }
 
@@ -382,42 +392,45 @@ init_upnp(const char *listenip, uint16_t listenport)
     int res;
     char *mediaserver_desc;
 
-    say(3, "Initializing UPnP subsystem...\n");
+    if (listenip != NULL)
+        say(2, _("Using IP address %s.\n"), quotearg(listenip));
+
+    say(3, _("Initializing UPnP subsystem...\n"));
     res = UpnpInit(listenip, listenport);
     if (res != UPNP_E_SUCCESS)
-        die("Cannot initialize UPnP subsystem - %s\n", upnp_errmsg(res));
+        die(_("Cannot initialize UPnP subsystem - %s\n"), upnp_errmsg(res));
 
-    say(1, "UPnP MediaServer listening on %s:%d\n", UpnpGetServerIpAddress(), UpnpGetServerPort());
+    say(1, _("UPnP MediaServer listening on %s:%d\n"), UpnpGetServerIpAddress(), UpnpGetServerPort());
 
-    say(3, "Enabling UPnP web server...\n");
+    say(3, _("Enabling UPnP web server...\n"));
     res = UpnpEnableWebserver(TRUE);
     if (res != UPNP_E_SUCCESS)
-        die("Cannot enable UPnP web server - %s\n", upnp_errmsg(res));
+        die(_("Cannot enable UPnP web server - %s\n"), upnp_errmsg(res));
     res = UpnpSetVirtualDirCallbacks(&virtual_dir_callbacks);
     if (res != UPNP_E_SUCCESS)
-        die("Cannot set virtual directory callbacks - %s\n", upnp_errmsg(res));
+        die(_("Cannot set virtual directory callbacks - %s\n"), upnp_errmsg(res));
     res = UpnpAddVirtualDir("/audio");
     if (res != UPNP_E_SUCCESS)
-        die("Cannot add virtual directory for web server - %s\n", upnp_errmsg(res));
+        die(_("Cannot add virtual directory for web server - %s\n"), upnp_errmsg(res));
 
-    say(3, "Registering UPnP root device...\n");
+    say(3, _("Registering UPnP root device...\n"));
 
-    if (friendly_name != NULL)
-	friendly_name = xasprintf("GMediaServer on %s", xgethostname());
+    if (friendly_name == NULL)
+	friendly_name = xasprintf(_("GMediaServer on %s"), xgethostname());
     mediaserver_desc = xasprintf(MEDIASERVER_DESC, friendly_name);
     free(friendly_name);
     res = UpnpRegisterRootDevice2(UPNPREG_BUF_DESC, mediaserver_desc,
               strlen(mediaserver_desc), 1, device_callback_event_handler, NULL, &device);
     free(mediaserver_desc);
     if (res != UPNP_E_SUCCESS)
-        die("Cannot register root device - %s\n", upnp_errmsg(res));
+        die(_("Cannot register root device - %s\n"), upnp_errmsg(res));
 
-    say(1, "Sending UPnP advertisement for device...\n");
+    say(1, _("Sending UPnP advertisement for device...\n"));
     res = UpnpSendAdvertisement(device, SSDP_PAUSE);
     if (res != UPNP_E_SUCCESS)
-        die("Cannot send device advertisement - %s\n", upnp_errmsg(res));
+        die(_("Cannot send device advertisement - %s\n"), upnp_errmsg(res));
 
-    say(1, "Listening for control point connections...\n");
+    say(1, _("Listening for control point connections...\n"));
 }
 
 void
@@ -425,13 +438,13 @@ finish_upnp(void)
 {
     int res;
 
-    say(1, "Shutting down UPnP MediaServer...\n");
+    say(1, _("Shutting down UPnP MediaServer...\n"));
     res = UpnpUnRegisterRootDevice(device);
     if (res != UPNP_E_SUCCESS)
-        die("Cannot unregister root device - %s\n", upnp_errmsg(res));
+        die(_("Cannot unregister root device - %s\n"), upnp_errmsg(res));
 
-    say(3, "Deinitializing UPnP subsystem...\n");
+    say(3, _("Deinitializing UPnP subsystem...\n"));
     res = UpnpFinish();
     if (res != UPNP_E_SUCCESS)
-        die("Cannot deinitialize UPnP library - %s\n", upnp_errmsg(res));
+        die(_("Cannot deinitialize UPnP library - %s\n"), upnp_errmsg(res));
 }
