@@ -19,10 +19,13 @@
  */
 
 #include <config.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <fcntl.h>
+#include <sys/types.h>		/* POSIX */
+#include <sys/stat.h>		/* ? */
+#include <unistd.h>		/* POSIX */
+#include <fcntl.h>		/* ? */
+#include "gettext.h"            /* Gnulib/gettext */
+#define _(s) gettext(s)
+#define N_(s) gettext_noop(s)
 #include "xalloc.h"
 #include "intutil.h"
 #include "gmediaserver.h"
@@ -59,7 +62,7 @@ webserver_get_info(const char *filename, struct File_Info *info)
     char *fullpath;
     struct stat sb;
 
-    say(4, "Request file information for %s\n", filename);
+    say(4, _("Request file information for %s\n"), filename);
 
     entry = get_file_entry(filename);
     if (entry == NULL)
@@ -67,14 +70,14 @@ webserver_get_info(const char *filename, struct File_Info *info)
 
     fullpath = get_entry_path(entry);
     if (stat(fullpath, &sb) < 0) {
-        warn("%s: cannot stat: %s\n", fullpath, errstr);
+        warn(_("%s: cannot stat: %s\n"), fullpath, errstr);
 	free(fullpath);
 	return -1;
     }
 
     if (access(fullpath, R_OK) < 0) {
 	if (errno != EACCES) {
-	    warn("%s: cannot check access: %s\n", fullpath, errstr);
+	    warn(_("%s: cannot check access: %s\n"), fullpath, errstr);
 	    free(fullpath);
 	    return -1;
 	}
@@ -93,7 +96,7 @@ webserver_get_info(const char *filename, struct File_Info *info)
 	info->content_type = ixmlCloneDOMString("audio/mpeg");
     }
 
-    say(4, "Returned info for file %s successfully\n", fullpath);
+    say(4, _("Returned info for file %s successfully\n"), fullpath);
 
     return 0;
 }
@@ -107,7 +110,7 @@ webserver_open(const char *filename, enum UpnpOpenFileMode mode)
     WebServerFile *file;
 
     if (mode != UPNP_READ) {
-	warn("%s: ignoring request to open file for writing\n", filename);
+	warn(_("%s: ignoring request to open file for writing\n"), filename);
 	return NULL;
     }
 
@@ -119,7 +122,7 @@ webserver_open(const char *filename, enum UpnpOpenFileMode mode)
 
     fd = open(fullpath, O_RDONLY);
     if (fd < 0) {
-	warn("%s: cannot open for reading: %s\n", fullpath, errstr);
+	warn(_("%s: cannot open for reading: %s\n"), fullpath, errstr);
 	free(fullpath);
 	return NULL;
     }
@@ -138,11 +141,11 @@ webserver_read(UpnpWebFileHandle fh, char *buf, size_t buflen)
     WebServerFile *file = (WebServerFile *) fh;
     ssize_t len;
 
-    say(4, "Attempting to read %d bytes from %s\n", buflen, file->fullpath);
+    say(4, _("Attempting to read %d bytes from %s\n"), buflen, file->fullpath);
 
     len = read(file->fd, buf, buflen);
     if (len < 0)
-	warn("%s: cannot read: %s\n", file->fullpath, errstr);
+	warn(_("%s: cannot read: %s\n"), file->fullpath, errstr);
 
     return len;
 }
@@ -152,7 +155,7 @@ webserver_write(UpnpWebFileHandle fh, char *buf, size_t buflen)
 {
     WebServerFile *file = (WebServerFile *) fh;
 
-    warn("%s: ignoring request to write to file\n", file->fullpath);
+    warn(_("%s: ignoring request to write to file\n"), file->fullpath);
 
     return -1;
 }
@@ -162,10 +165,10 @@ webserver_seek(UpnpWebFileHandle fh, long offset, int origin)
 {
     WebServerFile *file = (WebServerFile *) fh;
 
-    say(4, "Attempting to seek to %ld (origin %d) in %s\n", offset, origin, file->fullpath);
+    say(4, _("Attempting to seek to %ld (origin %d) in %s\n"), offset, origin, file->fullpath);
 
     if (lseek(file->fd, offset, origin) == -1) {
-	warn("%s: cannot seek: %s\n", file->fullpath, errstr);
+	warn(_("%s: cannot seek: %s\n"), file->fullpath, errstr);
 	return -1;
     }
 
